import { EventEmitter, NgZone } from '@angular/core';
import * as THREE from 'three';
import { Ref } from '../ref';
import { NgtStore } from '../stores/store';
import type { AnyFunction, BooleanInput, NgtColor, NgtEuler, NgtEvent, NgtQuaternion, NgtRenderState, NgtTriple, NgtVector3, NumberInput, UnknownRecord } from '../types';
import type { NgtInstanceState } from './instance';
import { NgtInstance } from './instance';
import * as i0 from "@angular/core";
export declare type NgtPreObjectInit = ((initFn: () => void) => void) | undefined;
export interface NgtObjectInputsState<TObject extends THREE.Object3D = THREE.Object3D> extends NgtInstanceState<TObject> {
    name: string;
    nameExplicit?: boolean;
    position: THREE.Vector3;
    positionExplicit?: boolean;
    rotation: THREE.Euler;
    rotationExplicit?: boolean;
    quaternion: THREE.Quaternion;
    quaternionExplicit?: boolean;
    scale: THREE.Vector3;
    scaleExplicit?: boolean;
    color: THREE.Color;
    colorExplicit?: boolean;
    castShadow: boolean;
    castShadowExplicit?: boolean;
    receiveShadow: boolean;
    receiveShadowExplicit?: boolean;
    priority: number;
    priorityExplicit?: boolean;
    visible: boolean;
    visibleExplicit?: boolean;
    matrixAutoUpdate: boolean;
    matrixAutoUpdateExplicit?: boolean;
    appendMode: 'immediate' | 'root' | 'none';
    appendModeExplicit?: boolean;
    userData?: UnknownRecord;
    userDataExplicit?: boolean;
    dispose?: (() => void) | null;
    disposeExplicit?: boolean;
    raycast?: THREE.Object3D['raycast'] | null;
    raycastExplicit?: boolean;
    appendTo?: Ref<THREE.Object3D>;
    appendToExplicit?: boolean;
}
export declare abstract class NgtObjectInputs<TObject extends THREE.Object3D = THREE.Object3D, TObjectInputsState extends NgtObjectInputsState<TObject> = NgtObjectInputsState<TObject>> extends NgtInstance<TObject, TObjectInputsState> {
    protected parentObjectRef: AnyFunction<Ref<THREE.Object3D>>;
    protected parentObjectHostRef: AnyFunction<Ref<THREE.Object3D>>;
    set name(name: string);
    get name(): string;
    set position(position: NgtVector3 | undefined);
    get position(): THREE.Vector3;
    get rotation(): NgtEuler | NgtTriple | undefined;
    set rotation(rotation: NgtEuler | NgtTriple | undefined);
    get quaternion(): NgtQuaternion | undefined;
    set quaternion(quaternion: NgtQuaternion | undefined);
    get scale(): NgtVector3 | undefined;
    set scale(scale: NgtVector3 | undefined);
    get color(): NgtColor | undefined;
    set color(color: NgtColor | undefined);
    get castShadow(): BooleanInput;
    set castShadow(value: BooleanInput);
    get receiveShadow(): BooleanInput;
    set receiveShadow(value: BooleanInput);
    get priority(): NumberInput;
    set priority(priority: NumberInput);
    get visible(): BooleanInput;
    set visible(visible: BooleanInput);
    get matrixAutoUpdate(): BooleanInput;
    set matrixAutoUpdate(matrixAutoUpdate: BooleanInput);
    get userData(): UnknownRecord | undefined;
    set userData(userData: UnknownRecord | undefined);
    get dispose(): (() => void) | null | undefined;
    set dispose(dispose: (() => void) | null | undefined);
    get raycast(): THREE.Object3D['raycast'] | null | undefined;
    set raycast(raycast: THREE.Object3D['raycast'] | null | undefined);
    get appendMode(): 'immediate' | 'root' | 'none';
    set appendMode(appendMode: 'immediate' | 'root' | 'none');
    get appendTo(): Ref<THREE.Object3D> | THREE.Object3D | undefined;
    set appendTo(appendTo: Ref<THREE.Object3D> | THREE.Object3D | undefined);
    click: EventEmitter<NgtEvent<MouseEvent>>;
    contextmenu: EventEmitter<NgtEvent<MouseEvent>>;
    dblclick: EventEmitter<NgtEvent<MouseEvent>>;
    pointerup: EventEmitter<NgtEvent<PointerEvent>>;
    pointerdown: EventEmitter<NgtEvent<PointerEvent>>;
    pointerover: EventEmitter<NgtEvent<PointerEvent>>;
    pointerout: EventEmitter<NgtEvent<PointerEvent>>;
    pointerenter: EventEmitter<NgtEvent<PointerEvent>>;
    pointerleave: EventEmitter<NgtEvent<PointerEvent>>;
    pointermove: EventEmitter<NgtEvent<PointerEvent>>;
    pointermissed: EventEmitter<NgtEvent<PointerEvent>>;
    pointercancel: EventEmitter<NgtEvent<PointerEvent>>;
    wheel: EventEmitter<NgtEvent<WheelEvent>>;
    shouldPassThroughRef: boolean;
    protected get optionFields(): Record<string, boolean>;
    constructor(zone: NgZone, store: NgtStore, parentObjectRef: AnyFunction<Ref<THREE.Object3D>>, parentObjectHostRef: AnyFunction<Ref<THREE.Object3D>>);
    static ɵfac: i0.ɵɵFactoryDeclaration<NgtObjectInputs<any, any>, [null, null, { optional: true; skipSelf: true; }, { optional: true; skipSelf: true; }]>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<NgtObjectInputs<any, any>, never, never, { "name": "name"; "position": "position"; "rotation": "rotation"; "quaternion": "quaternion"; "scale": "scale"; "color": "color"; "castShadow": "castShadow"; "receiveShadow": "receiveShadow"; "priority": "priority"; "visible": "visible"; "matrixAutoUpdate": "matrixAutoUpdate"; "userData": "userData"; "dispose": "dispose"; "raycast": "raycast"; "appendMode": "appendMode"; "appendTo": "appendTo"; }, { "click": "click"; "contextmenu": "contextmenu"; "dblclick": "dblclick"; "pointerup": "pointerup"; "pointerdown": "pointerdown"; "pointerover": "pointerover"; "pointerout": "pointerout"; "pointerenter": "pointerenter"; "pointerleave": "pointerleave"; "pointermove": "pointermove"; "pointermissed": "pointermissed"; "pointercancel": "pointercancel"; "wheel": "wheel"; }, never>;
}
export declare abstract class NgtObject<TObject extends THREE.Object3D = THREE.Object3D, TObjectState extends NgtObjectInputsState<TObject> = NgtObjectInputsState<TObject>> extends NgtObjectInputs<TObject, TObjectState> {
    appended: EventEmitter<TObject>;
    beforeRender: EventEmitter<{
        state: NgtRenderState;
        object: TObject;
    }>;
    /**
     * @deprecated Use {@link beforeRender} instead
     */
    animateReady: EventEmitter<{
        state: NgtRenderState;
        object: TObject;
    }>;
    protected abstract objectInitFn(): TObject;
    ngOnInit(): void;
    /**
     * Sub-classes can use this function to add additional logic BEFORE
     * initializing the object3d. The actual `initFn` is passed in so
     * the implementor needs to call this initFn() manually.
     *
     * This function is also called outside of Angular Zone
     * @protected
     */
    protected get preObjectInit(): NgtPreObjectInit;
    private readonly init;
    protected destroy(): void;
    private appendToParent;
    private addToScene;
    private addToParent;
    private shouldUseParent;
    private remove;
    private switch;
    private eventNameToHandler;
    static ɵfac: i0.ɵɵFactoryDeclaration<NgtObject<any, any>, never>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<NgtObject<any, any>, never, never, {}, { "appended": "appended"; "beforeRender": "beforeRender"; "animateReady": "animateReady"; }, never>;
}
